use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007000
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
Marcel
Gruenauer
marcel
Victor
Moral
victor
Ryan
Thompson
rct
at
thompsonclan
d0t
org
Aristotle
Pagaltzis
pagaltzis
Randy
Stauner
rwstauner
aristotle
Cindy
Wang
CindyLinz
cindylinz
Graham
Ollis
plicease
Mike
Covington
mfcovington
motemen
moznion
Peter
Vereshagin
veresc
rthompson
Yasutaka
ATARASHI
yakex
lib
Pod
Markdown
Perldoc
ToMarkdown
